#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <time.h>

#define FORTUNEFILE1 "./spacefortune"
#define FORTUNEFILE2 "/usr/games/lib/fortunes/spacefortune"

#define MAX_ENTRY   4096
#define MAX_LINE    256
#define MAXCOLS     255

#define UND 1
#define INT 4

static int Entries;
static char *epo[MAX_ENTRY];

static void Splittaa(char *s, int showcopyrights);
static void Dumppah(const char *s);

int main(int argc, const char **argv)
{
	char *s;
	off_t size;
	int fd;
	
	fd = open(FORTUNEFILE1, O_RDONLY);
	if(fd < 0)
	{
		fd = open(FORTUNEFILE2, O_RDONLY);
		if(fd < 0)
			if(argc <= 1)
			{
				fprintf(stderr, "spacefortune can't open it's data file.\n");
				return -1;
			}
	}
		
	argv=argv;	
	if(argc > 1)
		printf(
			"# \n"
			"# spacefortune is copyright (C) 1992,1999 Bisqwit (http://iki.fi/bisqwit/)\n"
			"# spacefortune is free for non-commercial use. No warranty.\n"
			"# Commercial users, contact bisqwit@iki.fi.\n");
	
	if(fd < 0)
		return -1;
	
	size = lseek(fd, 0, SEEK_END);
	
	s = (char *)malloc(size+1);
	if(!s)return -1;
	
	lseek(fd, 0, SEEK_SET);
	read(fd, s, size);
	close(fd);
	
	s[size] = 0;
	
	Splittaa(s, argc > 1);
	if(argc <= 1)
	{
		srand(time(NULL));
		Dumppah(epo[(int)((float)Entries * rand() / (RAND_MAX + 1.0))]);
	}	
	return 0;
}

static void Dumppah(const char *s)
{
	int COLS=80;
	int lonlin, n;
	const char *S;
	
#if defined(DJGPP) || defined(__BORLANDC__)
    struct text_info w;
    gettextinfo(&w);
    COLS = w.screenwidth;
#else
#ifdef TIOCGWINSZ
    struct winsize w;
    if(ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) >= 0)
        COLS = w.ws_col;
#else
#ifdef WIOCGETD
    struct uwdata w;
    if(ioctl(STDOUT_FILENO, WIOCGETD, &w) >= 0)
        COLS = w.uw_width / w.uw_hs;
#endif
#endif
#endif
	if(COLS > MAXCOLS)COLS = MAXCOLS;
	if(!COLS)COLS=74;
	
	lonlin = 0;
	
	for(S=s; *S; )
	{
		const char *t = S;
		int linlen, space, maxspace=COLS-3;
		for(space=maxspace; ;)
		{
			int viva=0, lev;
			const char *q = t;
			if(*t == '_')viva=1, t++;
			while(*t != ' ' && *t != '\n')t++;
			
			lev = (t-q) - viva + 1;
			while(*t == ' ')t++;
			if(space < lev)
			{
				t = q;
				break;
			}
			space -= lev;
			if(*t == '\n')
				break;
		}
		linlen=maxspace-space;
		if(linlen > lonlin)
			lonlin = linlen;		

		for(S=t; *S==' '||*S=='\n'; S++);
	}
	
	printf("\r\33[%dm+", UND);
	for(n=0; n<=lonlin; n++)putchar('-');
	printf("+\33[m\n");

	for(; *s; )
	{
		const char *t = s;
		int space, viivoja, intens, spaces;
		char *a, Buf[MAXCOLS+1];
		
		/* Sanojen haku stringist */		
		for(spaces=viivoja=0, space=lonlin; ;)
		{
			int viva=0, lev;
			const char *q = t;
			if(*t == '_')viva=1, t++, viivoja++;
			while(*t != ' ' && *t != '\n')t++;
			
			lev = (t-q) - viva + 1;
			while(*t == ' ')t++;
			if(space < lev)
			{
				t = q;
				break;
			}
			if(*t == '\n')
			{
				spaces=2;
				break;
			}
			space -= lev;
		}
		
		space = lonlin;
		
		/* Korostuksien ksittely */		
		a = Buf;
		for(intens=0; s<t; s++)
			if(*s=='_')
			{
				*a++=27;
				*a++='[';
				*a++=INT+'0';
				*a++='m';
				intens=1;
			}
			else
			{
				if(*s==' ' && intens)
				{
					intens=0;
					*a++=27;
					*a++='[';
					*a++='m';
				}
				*a++ = *s;
				if(*s==' ' && !spaces)
					spaces++;
				space--;
			}
			
		*a=0;
		
		if(spaces < 2)
		{
			/* Lasketaan spacejen mr merkkijonossa */
			for(spaces=n=0; Buf[n]; n++)
				if(Buf[n]==' ' && Buf[n+1])
					spaces++;
    	}
    	else
    	{
    		/* Nin tehdn */
    		spaces = 0;
    	}
    	
    	/* Jos merkkijonossa ei ollut spaceja */
    	if(!spaces)
		{
			/* Fillataan loppu tyhjksi */
			char *t = strchr(Buf, 0);
			memset(t, ' ', space);
			t[space] = 0;
		}
		else
		{
			/* Algoritmi: Jokaisen spacen kohdalla
			 * katsotaan kuinka monta spacea siihen
			 * pit list.
			 */
			float a=0.0, avgadd = space / (float)spaces;			
			for(n=0; spaces--; )
			{
				/* Muuten niit listn */
				while(Buf[n] != ' ')n++;
				
				/* Nyt n = spacen kohta */
				for(a+=avgadd; a>0.5; a-=1.0)
				{
  					memmove(Buf+n+1, Buf+n, strlen(Buf)+1-n);
   					Buf[n] = ' ';
				}
				while(Buf[n]==' ')n++;
			}
		}
			
		printf("\33[%dm|\33[m %s\33[0;%dm|\33[m\n", UND, Buf, UND);

		while(*s==' '||*s=='\n')s++;
	}
	
	printf("\33[%dm+", UND);
	for(n=0; n<=lonlin; n++)putchar('-');
	printf("+\33[m\n");
}

static void Splittaa(char *s, int showcopyrights)
{
	char *t;
	int yep;
	
	Entries=0;
	
	while(*s=='#')
	{
		for(; *s!='\n' && *s; s++)
			if(showcopyrights)
				putchar(*s);
		if(*s)
		{
			if(showcopyrights)
				putchar(*s);
			s++;
		}
	}
	while(*s=='\n')s++;
	if(showcopyrights)return;
	
	for(t=s, yep=1; *t; t++)
	{
		if(yep)
		{
			epo[Entries++] = t;
			yep=0;
		}		
		if(*t=='\n')
			if(t[1]=='\n')
			{
				*++t=0;
				yep=1;
			}
	}
}
